//
//  DataMonitorPluginController.m
//  GlowWorm
//
//  Created by Curtis Jones on 2007.02.12.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "DataMonitorPluginController.h"

#import <Common.h>
#import <Logger.h>
#import <GlowWorm.h>
#import <StatTrans.h>
#import <GlowWormStat.h>
#import <GlowWormPlugin.h>





@implementation DataMonitorPluginController





///////////////////////////////////////////////////////////////////////////////

#pragma mark -
#pragma mark initalization functions





- (id)init
{
	self = [super init];
	
#ifdef STANDALONE
	[GlowWorm getInstance];
#endif
	
	if (self) {
		mMonitorDict = [[NSMutableDictionary dictionaryWithCapacity:10] retain];
	}
	
	return self;
}

- (void)dealloc
{
	[super dealloc];
}

- (void)awakeFromNib
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	
	// load our sub-menu
	{
		mMenuMainMenu  = [[[NSMenu alloc] init] autorelease];
		mMenuMainItem  = [[[NSMenuItem alloc] init] autorelease];
		mMenuPrefsItem = [[[NSMenuItem alloc] init] autorelease];
	//mMenuSetupItem = [[[NSMenuItem alloc] init] autorelease];
		
		[mMenuMainItem setTitle:@"Data Monitor"];
		[mMenuMainItem setEnabled:TRUE];
		
		[mMenuPrefsItem setTarget:self];
		[mMenuPrefsItem setAction:@selector(actionPreferences:)];
		[mMenuPrefsItem setTitle:@"Preferences..."];
		[mMenuPrefsItem setEnabled:TRUE];
		
		/*
		[mMenuSetupItem setTarget:self];
		[mMenuSetupItem setAction:@selector(actionMonitor:)];
		[mMenuSetupItem setTitle:@"Monitor Window"];
		[mMenuSetupItem setEnabled:TRUE];
		*/
		
#ifndef STANDALONE
		[[GlowWormPlugin pluginsMenu] addItem:mMenuMainItem];
		[[GlowWormPlugin pluginsMenu] setSubmenu:mMenuMainMenu forItem:mMenuMainItem];
#else
		[[DataMonitorController alloc] initWithStatId:[NSNumber numberWithUnsignedLong:0xDEADBEEF]];
#endif
		
		[mMenuMainMenu addItem:mMenuPrefsItem];
	//[mMenuMainMenu addItem:mMenuRulesItem];
	}
	
	[self startup];
	
	[pool release];
}

- (void)startup
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
	//
	// register to be notified if any preference values 
	// change, and set the default values for those 
	// preferences if there is no current value.
	//
	{
		[[NSNotificationCenter defaultCenter] addObserver:self
																						 selector:@selector(actionPluginDisableCallback:)
																								 name:@"actionPluginDisable"
																							 object:nil];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
																						 selector:@selector(actionPluginEnableCallback:)
																								 name:@"actionPluginEnable"
																							 object:nil];
		
		/*
		if (nil == [[[NSUserDefaultsController sharedUserDefaultsController] values] valueForKey:@"showServiceNamesTrans"])
			[[[NSUserDefaultsController sharedUserDefaultsController] values] setValue:[NSNumber numberWithBool:TRUE] forKey:@"showServiceNamesTrans"];
		
		mPrefShowServiceNames = [[[[NSUserDefaultsController sharedUserDefaultsController] values] valueForKey:@"showServiceNamesTrans"  ] boolValue];
		
		[mRuleSrc showServiceNames:mPrefShowServiceNames];
		*/
	}
	
//[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(mainWindowSelection:) name:@"NSTableViewSelectionDidChangeNotification" object:mMainTbl];
	
	[self startupLoadRules];
	
//[mMainWin makeKeyAndOrderFront:nil];
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	GWLog(@"GlowWormController::observeValueForKeyPath:ofObject:change:context().. keyPath=%@", keyPath);
	
	/*
	// showServiceNamesTrans
	//
	if ([keyPath isEqualToString:@"values.showServiceNamesTrans"]) {
		mPrefShowServiceNames = [[[object values] valueForKey:@"showServiceNamesTrans"	] boolValue];
		[mRuleSrc showServiceNames:mPrefShowServiceNames];
		[mRuleTbl reloadData];
	}
	*/
	
}

- (void)startupLoadRules
{
#ifndef STANDALONE
	[GlowWormStat statInsertHandler:self callbackSel:@selector(statCallback:) callbackCtx:nil];
	[GlowWormRule insertRuleChangeHandlerForList:0 andTerm:GWR_TERM_PERM callbackObj:self callbackSel:@selector(ruleChangeCallback:)];
	[GlowWormRule selectRule:0 list:GWR_LIST_CONN term:GWR_TERM_PERM callbackObj:self callbackSel:@selector(ruleChangeCallback:)];
#endif
}

- (DataMonitorController *)newDataMonitorForRuleId:(NSNumber *)aRuleId
{
	if (aRuleId == nil)
		return nil;
	
	DataMonitorController * dmc = [[DataMonitorController alloc] initWithStatId:aRuleId];
	
	[mMonitorDict setObject:dmc forKey:aRuleId];
	
	return [dmc autorelease];
}





///////////////////////////////////////////////////////////////////////////////

#pragma mark -
#pragma mark action functions




/*
- (void)mainWindowSelection:(NSNotification *)aNotification
{
	GWLog(@"%s.. selected index = %d", __PRETTY_FUNCTION__, [mMainTbl selectedRow]);
	
	int row = [mMainTbl selectedRow];
	
	if (row < 0 || row >= [mDataSrc count])
		[mMainInfoTxt setEnabled:FALSE];
	else
		[mMainInfoTxt setEnabled:TRUE];
	
}
*/

- (IBAction)actionPreferences:(id)sender
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
//[mPrefWin makeKeyAndOrderFront:sender];
}





///////////////////////////////////////////////////////////////////////////////

#pragma mark -
#pragma mark observer callback functions





- (void)actionPluginDisableCallback:(NSNotification *)aNotification
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
//[mMainWin saveFrameUsingName:@"GlowWorm Plugin (Data Monitor)"];
//[mDataSrc delAll];
}

- (void)actionPluginEnableCallback:(NSNotification *)aNotification
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
	[self startupLoadRules];
}





///////////////////////////////////////////////////////////////////////////////

#pragma mark -
#pragma mark kext callback functions





- (void)statCallback:(StatTrans *)aTrans
{
	//GWLog(@"%s", __PRETTY_FUNCTION__);
	
	DataMonitorController * dmc = nil;
	GlowWormStat * stat = (GlowWormStat *)[aTrans body];
//GlowWormMesg * mesg = (GlowWormMesg *)[aTrans mesg];
	
	if (stat == nil) {
//	GWLog(@"%s.. handler confirmation callback; act3=0x%08lx", __PRETTY_FUNCTION__, [mesg act3]);
		return;
	}
	
	@synchronized (mMonitorDict)
	{
		dmc = [mMonitorDict objectForKey:[stat statId]];
		
		if (dmc == nil)
			dmc = [self newDataMonitorForRuleId:[stat statId]];
	}
	
	/*
	GWLog(@"%s.. [0x%08lx] id=%lu, bytes_send=%llu, pckts_send=%llu, bytes_recv=%llu, pckts_recv=%llu", __PRETTY_FUNCTION__, dmc,
				[[stat statId] unsignedLongValue],
				[[stat bytesSend] unsignedLongLongValue],
				[[stat pcktsSend] unsignedLongLongValue],
				[[stat bytesRecv] unsignedLongLongValue],
				[[stat pcktsRecv] unsignedLongLongValue]);
	*/
	
	[dmc addSample:stat];
}

- (void)ruleChangeCallback:(RuleTrans *)aTrans
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
	GlowWormRule * rule = (GlowWormRule *)[aTrans body];
//GlowWormMesg * mesg = (GlowWormMesg *)[aTrans mesg];
	
	if (rule == nil)
		return;
	
	NSNumber * statId = [rule uniqueId];
	NSNumber * stats = [rule stats];
	
	@synchronized (mMonitorDict)
	{
		DataMonitorController * dmc = [mMonitorDict objectForKey:statId];
		
		if (dmc != nil && (GWR_STATS_OFF == [stats intValue] || 0 == [stats intValue])) {
			[dmc stop];
			[mMonitorDict removeObjectForKey:statId];
		}
		else if (dmc != nil) {
			dmc->mMonitorView->mRate = [[rule statFreq] unsignedLongValue] * 1000000;
			
			if (dmc->mMonitorView->mRate == 0)
				dmc->mMonitorView->mRate = 1000000;
		}
		else if (dmc == nil && GWR_STATS_ON == [stats intValue]) {
			dmc = [self newDataMonitorForRuleId:statId];
			dmc->mMonitorView->mRate = [[rule statFreq] unsignedLongValue] * 1000000;
			
			if (dmc->mMonitorView->mRate == 0)
				dmc->mMonitorView->mRate = 1000000;
		}
	}
}





@end
